@@ -1,8 +1,8 @@
 #!/usr/bin/env python3
 
-# MIT License
+# Reticulum License
 #
-# Copyright (c) 2016-2022 Mark Qvist / unsigned.io
+# Copyright (c) 2020-2025 Mark Qvist
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -11,8 +11,16 @@
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
 #
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
+# - The Software shall not be used in any kind of system which includes amongst
+#   its functions the ability to purposefully do harm to human beings.
+#
+# - The Software shall not be used, directly or indirectly, in the creation of
+#   an artificial intelligence, machine learning or language model training
+#   dataset, including but not limited to any use that contributes to the
+#   training or development of such a model or algorithm.
+#
+# - The above copyright notice and this permission notice shall be included in
+#   all copies or substantial portions of the Software.
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -97,6 +105,11 @@ def apply_config():
         else:
             active_configuration["enable_propagation_node"] = False
 
+        if "propagation" in lxmd_config and "node_name" in lxmd_config["propagation"]:
+            active_configuration["node_name"] = lxmd_config["propagation"].get("node_name")
+        else:
+            active_configuration["node_name"] = None
+
         if "propagation" in lxmd_config and "auth_required" in lxmd_config["propagation"]:
             active_configuration["auth_required"] = lxmd_config["propagation"].as_bool("auth_required")
         else:
@@ -135,11 +148,58 @@ def apply_config():
                 active_configuration["propagation_transfer_max_accepted_size"] = 0.38
         else:
             active_configuration["propagation_transfer_max_accepted_size"] = 256
+
+        if "propagation" in lxmd_config and "propagation_message_max_accepted_size" in lxmd_config["propagation"]:
+            active_configuration["propagation_transfer_max_accepted_size"] = lxmd_config["propagation"].as_float("propagation_message_max_accepted_size")
+            if active_configuration["propagation_transfer_max_accepted_size"] < 0.38:
+                active_configuration["propagation_transfer_max_accepted_size"] = 0.38
+        else:
+            active_configuration["propagation_transfer_max_accepted_size"] = 256
+        
+        if "propagation" in lxmd_config and "propagation_sync_max_accepted_size" in lxmd_config["propagation"]:
+            active_configuration["propagation_sync_max_accepted_size"] = lxmd_config["propagation"].as_float("propagation_sync_max_accepted_size")
+            if active_configuration["propagation_sync_max_accepted_size"] < 0.38:
+                active_configuration["propagation_sync_max_accepted_size"] = 0.38
+        else:
+            active_configuration["propagation_sync_max_accepted_size"] = 256*40
+        
+        if "propagation" in lxmd_config and "propagation_stamp_cost_target" in lxmd_config["propagation"]:
+            active_configuration["propagation_stamp_cost_target"] = lxmd_config["propagation"].as_int("propagation_stamp_cost_target")
+            if active_configuration["propagation_stamp_cost_target"] < LXMF.LXMRouter.PROPAGATION_COST_MIN:
+                active_configuration["propagation_stamp_cost_target"] = LXMF.LXMRouter.PROPAGATION_COST_MIN
+        else:
+            active_configuration["propagation_stamp_cost_target"] = LXMF.LXMRouter.PROPAGATION_COST
+        
+        if "propagation" in lxmd_config and "propagation_stamp_cost_flexibility" in lxmd_config["propagation"]:
+            active_configuration["propagation_stamp_cost_flexibility"] = lxmd_config["propagation"].as_int("propagation_stamp_cost_flexibility")
+            if active_configuration["propagation_stamp_cost_flexibility"] < 0:
+                active_configuration["propagation_stamp_cost_flexibility"] = 0
+        else:
+            active_configuration["propagation_stamp_cost_flexibility"] = LXMF.LXMRouter.PROPAGATION_COST_FLEX
+        
+        if "propagation" in lxmd_config and "peering_cost" in lxmd_config["propagation"]:
+            active_configuration["peering_cost"] = lxmd_config["propagation"].as_int("peering_cost")
+            if active_configuration["peering_cost"] < 0:
+                active_configuration["peering_cost"] = 0
+        else:
+            active_configuration["peering_cost"] = LXMF.LXMRouter.PEERING_COST
+        
+        if "propagation" in lxmd_config and "remote_peering_cost_max" in lxmd_config["propagation"]:
+            active_configuration["remote_peering_cost_max"] = lxmd_config["propagation"].as_int("remote_peering_cost_max")
+            if active_configuration["remote_peering_cost_max"] < 0:
+                active_configuration["remote_peering_cost_max"] = 0
+        else:
+            active_configuration["remote_peering_cost_max"] = LXMF.LXMRouter.MAX_PEERING_COST
         
         if "propagation" in lxmd_config and "prioritise_destinations" in lxmd_config["propagation"]:
             active_configuration["prioritised_lxmf_destinations"] = lxmd_config["propagation"].as_list("prioritise_destinations")
         else:
             active_configuration["prioritised_lxmf_destinations"] = []
+        
+        if "propagation" in lxmd_config and "control_allowed" in lxmd_config["propagation"]:
+            active_configuration["control_allowed_identities"] = lxmd_config["propagation"].as_list("control_allowed")
+        else:
+            active_configuration["control_allowed_identities"] = []
 
         if "propagation" in lxmd_config and "static_peers" in lxmd_config["propagation"]:
             static_peers = lxmd_config["propagation"].as_list("static_peers")
@@ -323,10 +383,16 @@ def program_setup(configdir = None, rnsconfigdir = None, run_pn = False, on_inbo
         autopeer = active_configuration["autopeer"],
         autopeer_maxdepth = active_configuration["autopeer_maxdepth"],
         propagation_limit = active_configuration["propagation_transfer_max_accepted_size"],
+        propagation_cost = active_configuration["propagation_stamp_cost_target"],
+        propagation_cost_flexibility = active_configuration["propagation_stamp_cost_flexibility"],
+        peering_cost = active_configuration["peering_cost"],
+        max_peering_cost = active_configuration["remote_peering_cost_max"],
+        sync_limit = active_configuration["propagation_sync_max_accepted_size"],
         delivery_limit = active_configuration["delivery_transfer_max_accepted_size"],
         max_peers = active_configuration["max_peers"],
         static_peers = active_configuration["static_peers"],
-        from_static_only = active_configuration["from_static_only"])
+        from_static_only = active_configuration["from_static_only"],
+        name = active_configuration["node_name"])
     
     message_router.register_delivery_callback(lxmf_delivery)
 
@@ -359,13 +425,16 @@ def program_setup(configdir = None, rnsconfigdir = None, run_pn = False, on_inbo
         for dest_str in active_configuration["prioritised_lxmf_destinations"]:
             try:
                 dest_hash = bytes.fromhex(dest_str)
-                if len(dest_hash) == RNS.Reticulum.TRUNCATED_HASHLENGTH//8:
-                    message_router.prioritise(dest_hash)
-
-            except Exception as e:
-                RNS.log("Cannot prioritise "+str(dest_str)+", it is not a valid destination hash", RNS.LOG_ERROR)
+                if len(dest_hash) == RNS.Reticulum.TRUNCATED_HASHLENGTH//8: message_router.prioritise(dest_hash)
+            except Exception as e: RNS.log("Cannot prioritise "+str(dest_str)+", it is not a valid destination hash", RNS.LOG_ERROR)
 
         message_router.enable_propagation()
+        
+        for ident_str in active_configuration["control_allowed_identities"]:
+            try:
+                identity_hash = bytes.fromhex(ident_str)
+                if len(identity_hash) == RNS.Reticulum.TRUNCATED_HASHLENGTH//8: message_router.allow_control(identity_hash)
+            except Exception as e: RNS.log(f"Cannot allow control from {ident_str}, it is not a valid identity hash", RNS.LOG_ERROR)
 
         RNS.log("LXMF Propagation Node started on "+RNS.prettyhexrep(message_router.propagation_destination.hash))
 
@@ -416,14 +485,14 @@ def deferred_start_jobs():
     last_node_announce = time.time()
     threading.Thread(target=jobs, daemon=True).start()
 
-def query_status(identity, timeout=5, exit_on_fail=False):
-    control_destination = RNS.Destination(identity, RNS.Destination.OUT, RNS.Destination.SINGLE, APP_NAME, "propagation", "control")
+def _request_sync(identity, destination_hash, remote_identity, timeout=15, exit_on_fail=False):
+    control_destination = RNS.Destination(remote_identity, RNS.Destination.OUT, RNS.Destination.SINGLE, APP_NAME, "propagation", "control")
 
     timeout = time.time()+timeout
     def check_timeout():
         if time.time() > timeout:
             if exit_on_fail:
-                RNS.log("Getting lxmd statistics timed out, exiting now", RNS.LOG_ERROR)
+                print("Requesting lxmd peer sync timed out, exiting now")
                 exit(200)
             else:
                 return LXMF.LXMPeer.LXMPeer.ERROR_TIMEOUT
@@ -444,7 +513,7 @@ def query_status(identity, timeout=5, exit_on_fail=False):
             return tc
 
     link.identify(identity)
-    request_receipt = link.request(LXMF.LXMRouter.STATS_GET_PATH, data=None, response_callback=None, failed_callback=None)
+    request_receipt = link.request(LXMF.LXMRouter.SYNC_REQUEST_PATH, data=destination_hash, response_callback=None, failed_callback=None)
     while not request_receipt.get_status() == RNS.RequestReceipt.READY:
         tc = check_timeout()
         if tc:
@@ -453,60 +522,158 @@ def query_status(identity, timeout=5, exit_on_fail=False):
     link.teardown()
     return request_receipt.get_response()
 
-def get_status(configdir = None, rnsconfigdir = None, verbosity = 0, quietness = 0, timeout=5, show_status=False, show_peers=False, identity_path=None):
+
+def request_sync(target, remote=None, configdir=None, rnsconfigdir=None, verbosity=0, quietness=0, timeout=15, identity_path=None):
     global configpath, identitypath, storagedir, lxmdir
     global lxmd_config, active_configuration, targetloglevel
-    targetlogdest  = RNS.LOG_STDOUT
 
-    if identity_path == None:
-        if configdir == None:
-            if os.path.isdir("/etc/lxmd") and os.path.isfile("/etc/lxmd/config"):
-                configdir = "/etc/lxmd"
-            elif os.path.isdir(RNS.Reticulum.userdir+"/.config/lxmd") and os.path.isfile(Reticulum.userdir+"/.config/lxmd/config"):
-                configdir = RNS.Reticulum.userdir+"/.config/lxmd"
-            else:
-                configdir = RNS.Reticulum.userdir+"/.lxmd"
-
-        configpath   = configdir+"/config"
-        identitypath = configdir+"/identity"
-        identity = None
-
-        if not os.path.isdir(configdir):
-            RNS.log("Specified configuration directory does not exist, exiting now", RNS.LOG_ERROR)
-            exit(201)
-        if not os.path.isfile(identitypath):
-            RNS.log("Identity file not found in specified configuration directory, exiting now", RNS.LOG_ERROR)
-            exit(202)
-        else:
-            identity = RNS.Identity.from_file(identitypath)
-            if identity == None:
-                RNS.log("Could not load the Primary Identity from "+identitypath, RNS.LOG_ERROR)
-                exit(4)
-
-    else:
-        if not os.path.isfile(identity_path):
-            RNS.log("Identity file not found in specified configuration directory, exiting now", RNS.LOG_ERROR)
-            exit(202)
-        else:
-            identity = RNS.Identity.from_file(identity_path)
-            if identity == None:
-                RNS.log("Could not load the Primary Identity from "+identity_path, RNS.LOG_ERROR)
-                exit(4)        
-
-    if targetloglevel == None:
-        targetloglevel = 3
-    if verbosity != 0 or quietness != 0:
-        targetloglevel = targetloglevel+verbosity-quietness
-    
-    reticulum = RNS.Reticulum(configdir=rnsconfigdir, loglevel=targetloglevel, logdest=targetlogdest)
-    response = query_status(identity, timeout=timeout, exit_on_fail=True)
+    try:
+        peer_destination_hash = bytes.fromhex(target)
+        if len(peer_destination_hash) != RNS.Identity.TRUNCATED_HASHLENGTH//8: raise ValueError(f"Destination hash length must be {RNS.Identity.TRUNCATED_HASHLENGTH//8*2} characters")
+    except Exception as e:
+        print(f"Invalid peer destination hash: {e}")
+        exit(203)
+    remote
+    _remote_init(configdir, rnsconfigdir, verbosity, quietness, identity_path)
+    response = _request_sync(identity, peer_destination_hash, remote_identity=_get_target_identity(remote), timeout=timeout, exit_on_fail=True)
 
     if response == LXMF.LXMPeer.LXMPeer.ERROR_NO_IDENTITY:
-        RNS.log("Remote received no identity")
+        print("Remote received no identity")
+        exit(203)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_NO_ACCESS:
+        print("Access denied")
+        exit(204)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_INVALID_DATA:
+        print("Invalid data received by remote")
+        exit(205)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_NOT_FOUND:
+        print("The requested peer was not found")
+        exit(206)
+    elif response == None:
+        print("Empty response received")
+        exit(207)
+    else:
+        print(f"Sync requested for peer {RNS.prettyhexrep(peer_destination_hash)}")
+        exit(0)
+
+def _request_unpeer(identity, destination_hash, remote_identity, timeout=15, exit_on_fail=False):
+    control_destination = RNS.Destination(remote_identity, RNS.Destination.OUT, RNS.Destination.SINGLE, APP_NAME, "propagation", "control")
+
+    timeout = time.time()+timeout
+    def check_timeout():
+        if time.time() > timeout:
+            if exit_on_fail:
+                print("Requesting lxmd peering break timed out, exiting now")
+                exit(200)
+            else: return LXMF.LXMPeer.LXMPeer.ERROR_TIMEOUT
+        else: time.sleep(0.1)
+
+    if not RNS.Transport.has_path(control_destination.hash):
+        RNS.Transport.request_path(control_destination.hash)
+        while not RNS.Transport.has_path(control_destination.hash):
+            tc = check_timeout()
+            if tc:
+                return tc
+
+    link = RNS.Link(control_destination)
+    while not link.status == RNS.Link.ACTIVE:
+        tc = check_timeout()
+        if tc:
+            return tc
+
+    link.identify(identity)
+    request_receipt = link.request(LXMF.LXMRouter.UNPEER_REQUEST_PATH, data=destination_hash, response_callback=None, failed_callback=None)
+    while not request_receipt.get_status() == RNS.RequestReceipt.READY:
+        tc = check_timeout()
+        if tc:
+            return tc
+
+    link.teardown()
+    return request_receipt.get_response()
+
+
+def request_unpeer(target, remote=None, configdir=None, rnsconfigdir=None, verbosity=0, quietness=0, timeout=15, identity_path=None):
+    global configpath, identitypath, storagedir, lxmdir
+    global lxmd_config, active_configuration, targetloglevel
+
+    try:
+        peer_destination_hash = bytes.fromhex(target)
+        if len(peer_destination_hash) != RNS.Identity.TRUNCATED_HASHLENGTH//8: raise ValueError(f"Destination hash length must be {RNS.Identity.TRUNCATED_HASHLENGTH//8*2} characters")
+    except Exception as e:
+        print(f"Invalid peer destination hash: {e}")
+        exit(203)
+    remote
+    _remote_init(configdir, rnsconfigdir, verbosity, quietness, identity_path)
+    response = _request_unpeer(identity, peer_destination_hash, remote_identity=_get_target_identity(remote), timeout=timeout, exit_on_fail=True)
+
+    if response == LXMF.LXMPeer.LXMPeer.ERROR_NO_IDENTITY:
+        print("Remote received no identity")
+        exit(203)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_NO_ACCESS:
+        print("Access denied")
+        exit(204)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_INVALID_DATA:
+        print("Invalid data received by remote")
+        exit(205)
+    elif response == LXMF.LXMPeer.LXMPeer.ERROR_NOT_FOUND:
+        print("The requested peer was not found")
+        exit(206)
+    elif response == None:
+        print("Empty response received")
+        exit(207)
+    else:
+        print(f"Broke peering with {RNS.prettyhexrep(peer_destination_hash)}")
+        exit(0)
+
+def query_status(identity, remote_identity=None, timeout=5, exit_on_fail=False):
+    if remote_identity == None: remote_identity = identity
+    control_destination = RNS.Destination(remote_identity, RNS.Destination.OUT, RNS.Destination.SINGLE, APP_NAME, "propagation", "control")
+
+    timeout = time.time()+timeout
+    def check_timeout():
+        if time.time() > timeout:
+            if exit_on_fail:
+                print("Getting lxmd statistics timed out, exiting now")
+                exit(200)
+            else: return LXMF.LXMPeer.LXMPeer.ERROR_TIMEOUT
+        else: time.sleep(0.1)
+
+    if not RNS.Transport.has_path(control_destination.hash):
+        RNS.Transport.request_path(control_destination.hash)
+        while not RNS.Transport.has_path(control_destination.hash):
+            tc = check_timeout()
+            if tc: return tc
+
+    link = RNS.Link(control_destination)
+    while not link.status == RNS.Link.ACTIVE:
+        tc = check_timeout()
+        if tc: return tc
+
+    link.identify(identity)
+    request_receipt = link.request(LXMF.LXMRouter.STATS_GET_PATH, data=None, response_callback=None, failed_callback=None)
+    while not request_receipt.get_status() == RNS.RequestReceipt.READY:
+        tc = check_timeout()
+        if tc: return tc
+
+    link.teardown()
+    return request_receipt.get_response()
+
+def get_status(remote=None, configdir=None, rnsconfigdir=None, verbosity=0, quietness=0, timeout=5,
+               show_status=False, show_peers=False, identity_path=None):
+
+    global identity
+    _remote_init(configdir, rnsconfigdir, verbosity, quietness, identity_path)
+    response = query_status(identity, remote_identity=_get_target_identity(remote), timeout=timeout, exit_on_fail=True)
+
+    if response == LXMF.LXMPeer.LXMPeer.ERROR_NO_IDENTITY:
+        print("Remote received no identity")
         exit(203)
     if response == LXMF.LXMPeer.LXMPeer.ERROR_NO_ACCESS:
-        RNS.log("Access denied")
+        print("Access denied")
         exit(204)
+    elif response == None:
+        print("Empty response received")
+        exit(207)
     else:
         s = response
         mutil = round((s["messagestore"]["bytes"]/s["messagestore"]["limit"])*100, 2)
@@ -542,13 +709,19 @@ def get_status(configdir = None, rnsconfigdir = None, verbosity = 0, quietness =
         print(f"\nLXMF Propagation Node running on {dhs}, uptime is {uts}")
 
         if show_status:
-            msb = RNS.prettysize(s["messagestore"]["bytes"]); msl = RNS.prettysize(s["messagestore"]["limit"])
-            ptl = RNS.prettysize(s["propagation_limit"]*1000); uprx = RNS.prettysize(s["unpeered_propagation_rx_bytes"])
+            msb   = RNS.prettysize(s["messagestore"]["bytes"]); msl = RNS.prettysize(s["messagestore"]["limit"])
+            ptl   = RNS.prettysize(s["propagation_limit"]*1000); psl = RNS.prettysize(s["sync_limit"]*1000); 
+            uprx = RNS.prettysize(s["unpeered_propagation_rx_bytes"])
             mscnt = s["messagestore"]["count"]; stp = s["total_peers"]; smp = s["max_peers"]; sdp = s["discovered_peers"]
-            ssp = s["static_peers"]; cprr = s["clients"]["client_propagation_messages_received"]
-            cprs = s["clients"]["client_propagation_messages_served"]; upi = s["unpeered_propagation_incoming"]
+            ssp   = s["static_peers"]; cprr = s["clients"]["client_propagation_messages_received"]
+            cprs  = s["clients"]["client_propagation_messages_served"]; upi = s["unpeered_propagation_incoming"]
+            psc   = s["target_stamp_cost"]; scf = s["stamp_cost_flexibility"]
+            pc    = s["peering_cost"]; pcm = s["max_peering_cost"]
             print(f"Messagestore contains {mscnt} messages, {msb} ({ms_util} utilised of {msl})")
-            print(f"Accepting propagated messages from {who_str}, {ptl} per-transfer limit")
+            print(f"Required propagation stamp cost is {psc}, flexibility is {scf}")
+            print(f"Peering cost is {pc}, max remote peering cost is {pcm}")
+            print(f"Accepting propagated messages from {who_str}")
+            print(f"{ptl} message limit, {psl} sync limit")
             print(f"")
             print(f"Peers   : {stp} total (peer limit is {smp})")
             print(f"          {sdp} discovered, {ssp} static")
@@ -580,25 +753,110 @@ def get_status(configdir = None, rnsconfigdir = None, verbosity = 0, quietness =
                 h = max(time.time()-p["last_heard"], 0)
                 hops = p["network_distance"]
                 hs = "hops unknown" if hops == RNS.Transport.PATHFINDER_M else f"{hops} hop away" if hops == 1 else f"{hops} hops away"
-                pm = p["messages"]
+                pm = p["messages"]; pk = p["peering_key"]
+                pc = p["peering_cost"]; psc = p["target_stamp_cost"]; psf = p["stamp_cost_flexibility"]
+                if pc == None: pc = "unknown"
+                if psc == None: psc = "unknown"
+                if psf == None: psf = "unknown"
+                if pk == None: pk = "Not generated"
+                else:          pk = f"Generated, value is {pk}"
                 if p["last_sync_attempt"] != 0:
                     lsa = p["last_sync_attempt"]
                     ls = f"last synced {RNS.prettytime(max(time.time()-lsa, 0))} ago"
                 else:
                     ls = "never synced"
 
-                sstr = RNS.prettyspeed(p["str"]); sler = RNS.prettyspeed(p["ler"]); stl = RNS.prettysize(p["transfer_limit"]*1000)
+                sstr = RNS.prettyspeed(p["str"]); sler = RNS.prettyspeed(p["ler"])
+                stl = RNS.prettysize(p["transfer_limit"]*1000) if p["transfer_limit"] else "Unknown"
+                ssl = RNS.prettysize(p["sync_limit"]*1000) if p["sync_limit"] else "unknown"
                 srxb = RNS.prettysize(p["rx_bytes"]); stxb = RNS.prettysize(p["tx_bytes"]); pmo = pm["offered"]; pmout = pm["outgoing"]
-                pmi = pm["incoming"]; pmuh = pm["unhandled"]
+                pmi  = pm["incoming"]; pmuh = pm["unhandled"]; ar = round(p["acceptance_rate"]*100, 2)
+                if p["name"] == None: nn = ""
+                else: nn = p["name"].strip().replace("\n", "").replace("\r", "")
+                if len(nn) > 45: nn = f"{nn[:45]}..."
                 print(f"{ind}{t}{RNS.prettyhexrep(peer_id)}")
+                if len(nn): print(f"{ind*2}Name       : {nn}")
                 print(f"{ind*2}Status     : {a}, {hs}, last heard {RNS.prettytime(h)} ago")
-                print(f"{ind*2}Speeds     : {sstr} STR, {sler} LER, {stl} transfer limit")
-                print(f"{ind*2}Messages   : {pmo} offered, {pmout} outgoing, {pmi} incoming")
+                print(f"{ind*2}Costs      : Propagation {psc} (flex {psf}), peering {pc}")
+                print(f"{ind*2}Sync key   : {pk}")
+                print(f"{ind*2}Speeds     : {sstr} STR, {sler} LER")
+                print(f"{ind*2}Limits     : {stl} message limit, {ssl} sync limit")
+                print(f"{ind*2}Messages   : {pmo} offered, {pmout} outgoing, {pmi} incoming, {ar}% acceptance rate")
                 print(f"{ind*2}Traffic    : {srxb} received, {stxb} sent")
                 ms = "" if pm["unhandled"] == 1 else "s"
                 print(f"{ind*2}Sync state : {pmuh} unhandled message{ms}, {ls}")
                 print("")
 
+def _get_target_identity(remote=None, timeout=5):
+    global identity
+    timeout = time.time()+timeout
+    def check_timeout():
+        if time.time() > timeout:
+            print("Resolving remote identity timed out, exiting now")
+            exit(200)
+        else: time.sleep(0.1)
+
+    if remote == None: return identity
+    else:
+        try:
+            destination_hash = bytes.fromhex(remote)
+            if len(destination_hash) != RNS.Identity.TRUNCATED_HASHLENGTH//8: raise ValueError(f"Destination hash length must be {RNS.Identity.TRUNCATED_HASHLENGTH//8*2} characters")
+        except Exception as e:
+            print(f"Invalid remote destination hash: {e}")
+            exit(203)
+
+        remote_identity = RNS.Identity.recall(destination_hash)
+        if remote_identity: return remote_identity
+        else:
+            if not RNS.Transport.has_path(destination_hash):
+                RNS.Transport.request_path(destination_hash)
+                while not RNS.Transport.has_path(destination_hash):
+                    tc = check_timeout()
+                    if tc: return tc
+
+            return RNS.Identity.recall(destination_hash)
+
+def _remote_init(configdir=None, rnsconfigdir=None, verbosity=0, quietness=0, identity_path=None):
+    global configpath, identitypath, storagedir, lxmdir, identity
+    global lxmd_config, active_configuration, targetloglevel
+    targetlogdest  = RNS.LOG_STDOUT
+
+    if identity_path == None:
+        if configdir == None:
+            if os.path.isdir("/etc/lxmd") and os.path.isfile("/etc/lxmd/config"): configdir = "/etc/lxmd"
+            elif os.path.isdir(RNS.Reticulum.userdir+"/.config/lxmd") and os.path.isfile(Reticulum.userdir+"/.config/lxmd/config"): configdir = RNS.Reticulum.userdir+"/.config/lxmd"
+            else: configdir = RNS.Reticulum.userdir+"/.lxmd"
+
+        configpath   = configdir+"/config"
+        identitypath = configdir+"/identity"
+        identity = None
+
+        if not os.path.isdir(configdir):
+            RNS.log("Specified configuration directory does not exist, exiting now", RNS.LOG_ERROR)
+            exit(201)
+        if not os.path.isfile(identitypath):
+            RNS.log("Identity file not found in specified configuration directory, exiting now", RNS.LOG_ERROR)
+            exit(202)
+        else:
+            identity = RNS.Identity.from_file(identitypath)
+            if identity == None:
+                RNS.log("Could not load the Primary Identity from "+identitypath, RNS.LOG_ERROR)
+                exit(4)
+
+    else:
+        if not os.path.isfile(identity_path):
+            RNS.log("Identity file not found in specified configuration directory, exiting now", RNS.LOG_ERROR)
+            exit(202)
+        else:
+            identity = RNS.Identity.from_file(identity_path)
+            if identity == None:
+                RNS.log("Could not load the Primary Identity from "+identity_path, RNS.LOG_ERROR)
+                exit(4)        
+
+    if targetloglevel == None: targetloglevel = 3
+    if verbosity != 0 or quietness != 0: targetloglevel = targetloglevel+verbosity-quietness
+    
+    reticulum = RNS.Reticulum(configdir=rnsconfigdir, loglevel=targetloglevel, logdest=targetlogdest)
 
 def main():
     try:
@@ -612,8 +870,11 @@ def main():
         parser.add_argument("-s", "--service", action="store_true", default=False, help="lxmd is running as a service and should log to file")
         parser.add_argument("--status", action="store_true", default=False, help="display node status")
         parser.add_argument("--peers", action="store_true", default=False, help="display peered nodes")
-        parser.add_argument("--timeout", action="store", default=5, help="timeout in seconds for query operations", type=float)
-        parser.add_argument("--identity", action="store", default=None, help="path to identity used for query request", type=str)
+        parser.add_argument("--sync", action="store", default=None, help="request a sync with the specified peer", type=str)
+        parser.add_argument("-b", "--break", dest="unpeer", action="store", default=None, help="break peering with the specified peer", type=str)
+        parser.add_argument("--timeout", action="store", default=None, help="timeout in seconds for query operations", type=float)
+        parser.add_argument("-r", "--remote", action="store", default=None, help="remote propagation node destination hash", type=str)
+        parser.add_argument("--identity", action="store", default=None, help="path to identity used for remote requests", type=str)
         parser.add_argument("--exampleconfig", action="store_true", default=False, help="print verbose configuration example to stdout and exit")
         parser.add_argument("--version", action="version", version="lxmd {version}".format(version=__version__))
         
@@ -624,14 +885,40 @@ def main():
             exit()
 
         if args.status or args.peers:
+            if not args.timeout: args.timeout = 5
             get_status(configdir = args.config,
-                      rnsconfigdir=args.rnsconfig,
-                      verbosity=args.verbose,
-                      quietness=args.quiet,
-                      timeout=args.timeout,
-                      show_status=args.status,
-                      show_peers=args.peers,
-                      identity_path=args.identity)
+                       rnsconfigdir=args.rnsconfig,
+                       verbosity=args.verbose,
+                       quietness=args.quiet,
+                       timeout=args.timeout,
+                       show_status=args.status,
+                       show_peers=args.peers,
+                       identity_path=args.identity,
+                       remote=args.remote)
+            exit()
+
+        if args.sync:
+            if not args.timeout: args.timeout = 10
+            request_sync(target=args.sync,
+                         configdir = args.config,
+                         rnsconfigdir=args.rnsconfig,
+                         verbosity=args.verbose,
+                         quietness=args.quiet,
+                         timeout=args.timeout,
+                         identity_path=args.identity,
+                         remote=args.remote)
+            exit()
+
+        if args.unpeer:
+            if not args.timeout: args.timeout = 10
+            request_unpeer(target=args.unpeer,
+                           configdir = args.config,
+                           rnsconfigdir=args.rnsconfig,
+                           verbosity=args.verbose,
+                           quietness=args.quiet,
+                           timeout=args.timeout,
+                           identity_path=args.identity,
+                           remote=args.remote)
             exit()
 
         program_setup(configdir = args.config,
@@ -656,6 +943,17 @@ __default_lxmd_config__ = """# This is an example LXM Daemon config file.
 
 enable_node = no
 
+# You can specify identity hashes for remotes
+# that are allowed to control and query status
+# for this propagation node.
+
+# control_allowed = 7d7e542829b40f32364499b27438dba8, 437229f8e29598b2282b88bad5e44698
+
+# An optional name for this node, included
+# in announces.
+
+# node_name = Anonymous Propagation Node
+
 # Automatic announce interval in minutes.
 # 6 hours by default.
 
@@ -675,19 +973,6 @@ autopeer = yes
 
 autopeer_maxdepth = 4
 
-# The maximum accepted transfer size per in-
-# coming propagation transfer, in kilobytes.
-# This also sets the upper limit for the size
-# of single messages accepted onto this node.
-#
-# If a node wants to propagate a larger number
-# of messages to this node, than what can fit
-# within this limit, it will prioritise sending
-# the smallest messages first, and try again
-# with any remaining messages at a later point.
-
-propagation_transfer_max_accepted_size = 256
-
 # The maximum amount of storage to use for
 # the LXMF Propagation Node message store,
 # specified in megabytes. When this limit
@@ -700,6 +985,54 @@ propagation_transfer_max_accepted_size = 256
 
 # message_storage_limit = 500
 
+# The maximum accepted transfer size per in-
+# coming propagation message, in kilobytes.
+# This sets the upper limit for the size of
+# single messages accepted onto this node.
+
+# propagation_message_max_accepted_size = 256
+
+# The maximum accepted transfer size per in-
+# coming propagation node sync.
+#
+# If a node wants to propagate a larger number
+# of messages to this node, than what can fit
+# within this limit, it will prioritise sending
+# the smallest messages first, and try again
+# with any remaining messages at a later point.
+
+# propagation_sync_max_accepted_size = 10240
+
+# You can configure the target stamp cost
+# required to deliver messages via this node.
+
+# propagation_stamp_cost_target = 16
+
+# If set higher than 0, the stamp cost flexi-
+# bility option will make this node accept
+# messages with a lower stamp cost than the
+# target from other propagation nodes (but
+# not from peers directly). This allows the
+# network to gradually adjust stamp cost.
+
+# propagation_stamp_cost_flexibility = 3
+
+# The peering_cost option configures the target
+# value required for a remote node to peer with
+# and deliver messages to this node.
+
+# peering_cost = 18
+
+# You can configure the maximum peering cost
+# of remote nodes that this node will peer with.
+# Setting this to a higher number will allow
+# this node to peer with other nodes requiring
+# a higher peering key value, but will require
+# more computation time during initial peering
+# when generating the peering key.
+
+# remote_peering_cost_max = 26
+
 # You can tell the LXMF message router to
 # prioritise storage for one or more
 # destinations. If the message store reaches
@@ -712,9 +1045,9 @@ propagation_transfer_max_accepted_size = 256
 
 # You can configure the maximum number of other
 # propagation nodes that this node will peer
-# with automatically. The default is 50.
+# with automatically. The default is 20.
 
-# max_peers = 25
+# max_peers = 20
 
 # You can configure a list of static propagation
 # node peers, that this node will always be
